
package de.csw.ludum.dare.ld23;

import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferStrategy;
import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.JFrame;

import de.csw.ludum.dare.ld23.control.InputHandler;
import de.csw.ludum.dare.ld23.control.InputKey;
import de.csw.ludum.dare.ld23.graphic.fonts.BitmapFont;
import de.csw.ludum.dare.ld23.graphic.objects.BackGroundA;
import de.csw.ludum.dare.ld23.graphic.objects.BackGroundB;
import de.csw.ludum.dare.ld23.graphic.objects.DecoRenderables;
import de.csw.ludum.dare.ld23.graphic.objects.Hero;
import de.csw.ludum.dare.ld23.graphic.objects.enemies.Enemy;
import de.csw.ludum.dare.ld23.graphic.ui.End;
import de.csw.ludum.dare.ld23.graphic.ui.HealthBar;
import de.csw.ludum.dare.ld23.graphic.ui.HungerBar;
import de.csw.ludum.dare.ld23.graphic.ui.Splash;
import de.csw.ludum.dare.ld23.graphic.ui.Win;
import de.csw.ludum.dare.ld23.level.Level;
import de.csw.ludum.dare.ld23.sound.SoundEngine;
import de.csw.ludum.dare.ld23.util.BitmapFill;
import de.csw.ludum.dare.ld23.util.CheapColor;
import de.csw.ludum.dare.ld23.util.DebugLogger;

public class GameComponent extends Canvas implements Runnable {

  private static final long    serialVersionUID   = 1L;

  /// Statics ///

  public static final int      GAME_SMALL_WIDTH   = 160;                       // 512
  public static final int      GAME_SMALL_HEIGHT  = 90;

  public static final int      SCALE              = 6;

  public static final int      GAME_SCALED_WIDTH  = GAME_SMALL_WIDTH * SCALE;  // 512
  public static final int      GAME_SCALED_HEIGHT = GAME_SMALL_HEIGHT * SCALE;

  private static final long    TICKS_PER_SECOND   = 100;

  private static final long    TIME_PER_TICK      = 1000 / TICKS_PER_SECOND;

  SoundEngine                  soundEngine;

  private Thread               meThread;

  private boolean              running            = true;

  private GameScreen           screen;

  private Timer                timer;

  private static GameComponent gameComponent;

  public static long           tickCount          = 0;
  long                         lastTickTime       = System.currentTimeMillis();

  long                         lastRenderTime;
  long                         frameCounter;
  long                         fps;
  Hero                         hero;
  BackGroundA                  backGroundA;
  BackGroundB                  backGroundB;
  Level                        level;
  HealthBar                    healthBar;
  Splash                       splash;
  HungerBar                    hungerBar;
  DecoRenderables              decoRenderables;

  End                          end;
  Win                          win;

  public static GameComponent getInstance() {
    if (gameComponent == null) {
      gameComponent = new GameComponent();

      gameComponent.init();
    }
    return gameComponent;
  }

  public static void main(String[] args) {
    JFrame frame = new JFrame("LudumDareGame");
    GameComponent gameComponent = getInstance();

    frame.setLocation(3, 3);
    //    frame.setLocationRelativeTo(null);

    frame.setIgnoreRepaint(true);
    frame.setLayout(new BorderLayout());
    frame.add(gameComponent, BorderLayout.CENTER);
    frame.setResizable(false);
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.setVisible(true);
    frame.setSize(GAME_SCALED_WIDTH, GAME_SCALED_HEIGHT);
    frame.pack();

    gameComponent.start();
  }

  public void init() {
    lastRenderTime = System.currentTimeMillis();
    frameCounter = 0;
    fps = 0;
    hero = new Hero();
    backGroundA = new BackGroundA();
    backGroundB = new BackGroundB();
    level = new Level();
    healthBar = new HealthBar();
    end = new End();
    splash = new Splash();
    decoRenderables = new DecoRenderables();

    hungerBar = new HungerBar();
    GameOptions.init();

    screen = new GameScreen();
    win = new Win();
    running = true;
    //getInstance().start();

    SoundEngine.getInstance().init();
    SoundEngine.getInstance().song1.play();
  }

  public void start() {

    System.out.println("START");
    checkAndForceBufferStrategy();

    requestFocus();

    createThread();
  }

  // Evtl überschreiben?
  //  @Override
  //  public void paint(Graphics g) {
  //  }
  //
  //  @Override
  //  public void update(Graphics g) {
  //  }

  public void stop() {
    running = false;
  }

  public boolean isRunning() {
    return running;
  }

  private void createThread() {
    meThread = new Thread(this);
    meThread.setPriority(Thread.MAX_PRIORITY);
    meThread.start();
  }

  private void checkAndForceBufferStrategy() {
    BufferStrategy bs = getBufferStrategy();
    if (bs == null) {
      createBufferStrategy(3);
    }
  }

  public GameComponent() {
    this.setPreferredSize(new Dimension(GAME_SCALED_WIDTH, GAME_SCALED_HEIGHT));
    this.setMinimumSize(new Dimension(GAME_SCALED_WIDTH, GAME_SCALED_HEIGHT));
    this.setMaximumSize(new Dimension(GAME_SCALED_WIDTH, GAME_SCALED_HEIGHT));

    // TODO
    this.addKeyListener(new InputHandler());
  }

  // This is Active Rendering - http://www.gamedev.net/page/resources/_/technical/general-programming/java-games-active-rendering-r2418
  @Override
  public void run() {

    timer = new Timer();
    timer.schedule(new TimerTask() {

      @Override
      public void run() {
        try {
          tick();
        } catch (Exception e) {
          e.printStackTrace();
          throw new RuntimeException("Error in GameLoop!", e);
        }
      }
    }, 0, TIME_PER_TICK);

    long lastRenderTime = System.currentTimeMillis();

    while (isRunning()) {

      BufferStrategy bs = getBufferStrategy();

      Graphics g = bs.getDrawGraphics();

      render(g);
      try {
        long sleepTimeInMs = 20 - (System.currentTimeMillis() - lastRenderTime);
        if (sleepTimeInMs > 0) {
          Thread.sleep(sleepTimeInMs);
        }
        lastRenderTime = System.currentTimeMillis();
      } catch (InterruptedException e) {
        e.printStackTrace();
      }

      if (!bs.contentsLost()) {
        g.dispose();
        bs.show();
        countFrames();
      }
    }
  }

  private void render(Graphics g) {

    if (GameOptions.showSplash) {
      backGroundA.renderMe(screen);
      splash.renderMe(screen);

      screen.drawScreen(this, g);
      return;
    } else if (GameOptions.showWin) {
      win.renderMe(screen);
      screen.drawScreen(this, g);
      return;
    } else if (GameOptions.showEnd) {
      end.renderMe(screen);
      screen.drawScreen(this, g);
      return;
    }

    // TODO: Remove, not needed for finished game
    // screen.clear();

    BitmapFill.fillRectangle(screen, 0, 0, screen.width, screen.height, CheapColor.BLACK);

    backGroundA.renderMe(screen);
    backGroundB.renderMe(screen);

    // Render Level first!
    level.renderMe(screen, hero);

    List<Enemy> toRender = new ArrayList<Enemy>(level.enemies);
    for (Enemy enemy : toRender) {
      enemy.renderMe(screen, level, hero);
    }

    decoRenderables.renderMe(screen, level, hero);
    hero.renderMe(screen, level, hero);
    healthBar.renderMe(screen, level, hero);
    hungerBar.renderMe(screen, level, hero);

    if (GameOptions.SHOW_DEBUG_STATUS) {
      BitmapFont.draw(screen, "POS: " + hero.getPositionX() + ":" + hero.getPositionY(), 138 * SCALE, screen.height - 61);
      BitmapFont.draw(screen, "FPS: " + fps, 138 * SCALE, screen.height - 47);
      BitmapFont.draw(screen, "???", 138 * SCALE, screen.height - 19);
      BitmapFont.draw(screen, "Blit: " + hero.blitToX + ":" + hero.blitToY, 138 * SCALE, screen.height - 33);
      BitmapFont.draw(screen, "" + DebugLogger.debugMessage, 2, screen.height - 10);
    }

    screen.drawScreen(this, g);
  }

  private void countFrames() {

    frameCounter++;

    if (frameCounter % 100 == 0) {
      //      Logger.logInfoWithTimeAndThread("Counting Frames!");
      fps = 100000 / (System.currentTimeMillis() - lastRenderTime);
      //      System.out.println("COUNT FRAMES! [fps=" + this.fps + ",timeInMs=" + (System.currentTimeMillis() - this.lastRenderTime) + "]");
      lastRenderTime = System.currentTimeMillis();
    }
  }

  //  private void tickIfNeed() {
  //    if (System.currentTimeMillis() - lastTickTime > TIME_PER_TICK) {
  //      lastTickTime = System.currentTimeMillis();
  //      tick();
  //    }
  //  }

  private void tick() {

    tickCount++;

    InputKey.tickAllKeys();

    if (InputKey.SOUND.isDown) {
      SoundEngine.muteOrUnmute();
      try {
        Thread.sleep(250);
      } catch (InterruptedException e) {
        e.printStackTrace();
      }
    }

    if (GameOptions.showSplash) {
      splash.tick();
      backGroundA.offsetX -= 0.200d;
      return;
    } else if (GameOptions.showWin) {
      win.tick();
      return;
    } else if (GameOptions.showEnd) {
      end.tick();
      return;
    }

    backGroundB.offsetX -= 0.1d;
    backGroundA.offsetX -= 0.02d;

    hero.tick(level);
    level.tick(hero);
    decoRenderables.tick();

    List<Enemy> enemiesToRemove = new ArrayList<Enemy>();

    for (Enemy enemy : level.enemies) {

      enemy.tick(level, hero);

      if (hero.intersectsWithEnemy(enemy)) {
        if (hero.canEat(enemy)) {
          hero.eat(enemy);
          enemiesToRemove.add(enemy);
        } else {
          // TODO GET HURT
          hero.hitBy(enemy);

          if (enemy.eatableBy == null) {
            enemiesToRemove.add(enemy);
          }

          checkHeroHealth(hero);
        }
      }

      if (enemy.eatableBy == null) {
        if (lastTickTime - enemy.createAt > 10000) {
          enemiesToRemove.add(enemy);
        } else if (enemy.toRemove) {
          enemiesToRemove.add(enemy);
        }
      }
    }
    for (Enemy enemy : enemiesToRemove) {
      level.enemies.remove(enemy);
    }
    for (Enemy enemy : level.newEnemies) {
      level.enemies.add(enemy);
    }
    level.newEnemies.clear();
  }

  private void checkHeroHealth(Hero hero2) {
    if ((hero2.health == 0) && GameOptions.CAN_DIE) {
      GameOptions.showEnd = true;
    }
  }

  public static void restart() {
    getInstance().timer.cancel();
    getInstance().running = false;
    getInstance().init();
    try {
      Thread.sleep(50);
    } catch (InterruptedException e) {
      e.printStackTrace();
    }
    getInstance().createThread();
  }

}
