
package de.csw.ludum.dare.ld23.graphic;

import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import javax.imageio.ImageIO;

import de.csw.ludum.dare.ld23.GameComponent;
import de.csw.ludum.dare.ld23.util.BitmapTransform;
import de.csw.ludum.dare.ld23.util.DebugLogger;

public class ImageLoader {

  private ImageLoader() {
  }

  //URL url = new URL(getCodeBase(),
  //"strawberry.jpg");
  //img = ImageIO.read(url);

  public static Bitmap loadScaledBitmap(String path) {
    Bitmap bitmap = loadBitmap(path);
    return BitmapTransform.editAndScaleBitmap(bitmap, GameComponent.SCALE);
  }

  public static Bitmap loadBitmap(String path) {
    return new Bitmap(loadImage(path));
  }

  public static BufferedImage loadImage(String fullFilePath) {
    try {
      return ImageIO.read(getInputStreamForRessource(fullFilePath));
    } catch (IOException e) {
      throw new RuntimeException("Couldn't load image! [fullFilePath=" + fullFilePath + "]", e);
    }
  }

  private static InputStream getInputStreamForRessource(String fullFilePath) {
    URL url = ImageLoader.class.getResource(fullFilePath);
    DebugLogger.logInfo("URL is: " + url);
    InputStream result = ImageLoader.class.getResourceAsStream(fullFilePath);
    if (result == null) {
      throw new RuntimeException("Couldn't load image! [fullFilePath=" + fullFilePath + ",url=" + url + "]");
    }
    return result;
  }

  public static Sprite loadSprite(String path, int width, int height) {
    return loadSprite(path, width, height, 0, 0);
  }

  public static Sprite loadSprite(String path, int w, int h, int bx, int by) {
    BufferedImage bi = loadImage(path);

    int xTiles = (bi.getWidth() - bx) / w;
    int yTiles = (bi.getHeight() - by) / h;

    Bitmap[][] result = new Bitmap[xTiles][yTiles];

    for (int x = 0; x < xTiles; x++) {
      for (int y = 0; y < yTiles; y++) {
        result[x][y] = new Bitmap(w, h);
        bi.getRGB(bx + x * w, by + y * h, w, h, result[x][y].pixels, 0, w);
      }
    }

    return new Sprite(result, 12);
  }

  public static Sprite loadScaledSprite(String string, int i, int j, int scale) {
    return loadScaledSprite(string, i, j, scale, 12);
  }

  public static Sprite loadScaledSprite(String string, int i, int j, int scale, int ticksPerAnim) {
    Bitmap[][] result = loadSprite(string, i, j).result;

    for (int k = 0; k < result.length; k++) {
      for (int k2 = 0; k2 < result[k].length; k2++) {
        result[k][k2] = BitmapTransform.editAndScaleBitmap(result[k][k2], scale);
      }
    }

    return new Sprite(result, ticksPerAnim);
  }

}
