
package de.csw.ludum.dare.ld23.graphic.objects;

import de.csw.ludum.dare.ld23.GameComponent;
import de.csw.ludum.dare.ld23.GameScreen;
import de.csw.ludum.dare.ld23.graphic.Bitmap;
import de.csw.ludum.dare.ld23.graphic.ImageLoader;
import de.csw.ludum.dare.ld23.graphic.Renderable;
import de.csw.ludum.dare.ld23.graphic.Sprite;
import de.csw.ludum.dare.ld23.level.Level;

public class DecoRenderable extends Renderable {

  private final Sprite sprite;

  private final int    positionX;

  private final int    positionY;

  private int          ticksPerAnimation = 10;

  private int          currentFrame      = 0;

  public DecoRenderable(int posX, int posY, String filePath, int width, int height, int ticksPerAnimation) {
    sprite = ImageLoader.loadScaledSprite(filePath, width, height, GameComponent.SCALE);
    positionX = posX;
    positionY = posY;
    this.ticksPerAnimation = ticksPerAnimation;
  }

  @Override
  public void renderMe(GameScreen screen, Level level, Hero hero) {

    int blitToX = positionX + level.viewX;
    int blitToY = positionY + level.viewY;

    screen.blit(getCurrentBitmapFrame(), blitToX, blitToY);
  }

  private Bitmap getCurrentBitmapFrame() {

    return sprite.getBitmapDirect(0, currentFrame);
  }

  public void tick() {
    if (GameComponent.tickCount % ticksPerAnimation == 0) {
      increaseFrame();
    }
  }

  private void increaseFrame() {
    currentFrame++;
    if (currentFrame >= sprite.lines) {
      currentFrame = 0;
    }
  }

}
