
package de.csw.ludum.dare.ld23.graphic.objects;

import java.awt.Rectangle;

import de.csw.ludum.dare.ld23.GameComponent;
import de.csw.ludum.dare.ld23.GameOptions;
import de.csw.ludum.dare.ld23.GameScreen;
import de.csw.ludum.dare.ld23.control.InputKey;
import de.csw.ludum.dare.ld23.graphic.Bitmap;
import de.csw.ludum.dare.ld23.graphic.ImageLoader;
import de.csw.ludum.dare.ld23.graphic.Renderable;
import de.csw.ludum.dare.ld23.graphic.Sprite;
import de.csw.ludum.dare.ld23.graphic.objects.enemies.Enemy;
import de.csw.ludum.dare.ld23.level.Level;
import de.csw.ludum.dare.ld23.sound.SoundEngine;
import de.csw.ludum.dare.ld23.util.BitmapLine;
import de.csw.ludum.dare.ld23.util.CheapColor;
import de.csw.ludum.dare.ld23.util.Vector2D;

public class Hero extends Renderable {

  public static enum HeroTier {
    SMALL_RAT(2, 0.6f, 26),
    MEDIUM_RAT(6, 0.9f, 150),
    GIANT_RAT(12, 1.4f, 300);

    public final int   maxHealth;

    public final int   maxEatToGrow;

    public final float minVelocity;

    public final float maxVelocity;

    private HeroTier(int maxHealth, float maxVelocity, int maxEatToGrow) {
      this.maxHealth = maxHealth;
      this.maxEatToGrow = maxEatToGrow;

      minVelocity = maxVelocity * -1.0f;
      this.maxVelocity = maxVelocity;
    }
  }

  public int                 enemiesByHungerEaten  = 0;

  public HeroTier            heroTier              = HeroTier.SMALL_RAT;

  private static final float ACCELERATION_PER_TICK = 0.1f;

  public static final int    WIDTH                 = GameComponent.SCALE * 32;

  public static final int    HEIGHT                = GameComponent.SCALE * 32;

  private Sprite             sprite;

  private final Sprite       spriteSmall;

  private final Sprite       spriteMedium;

  private final Sprite       spriteGiant;

  public float               positionX             = 750;

  public float               positionY             = 880;

  private float              velocityX             = 0.0f;

  private float              velocityY             = 0.0f;

  private Rectangle          rectangle;
  private final Rectangle    rectangleS;
  private final Rectangle    rectangleM;
  private final Rectangle    rectangleL;
  public int                 blitToX               = 0;

  public int                 blitToY               = 0;

  public int                 health                = 2;

  public static final int    HERO_PIXEL_X_LEFT     = (GameComponent.GAME_SCALED_WIDTH / 2) - (WIDTH / 2);
  public static final int    heroPixelXRight       = (GameComponent.GAME_SCALED_WIDTH / 2) + (WIDTH / 2);

  public static final int    heroPixelYTop         = (GameComponent.GAME_SCALED_HEIGHT / 2) - (HEIGHT / 2);
  public static final int    heroPixelYBottom      = (GameComponent.GAME_SCALED_HEIGHT / 2) + (HEIGHT / 2);

  public Hero() {

    spriteSmall = ImageLoader.loadScaledSprite("/hero/heroS.png", 8, 5, GameComponent.SCALE);
    spriteMedium = ImageLoader.loadScaledSprite("/hero/heroM.png", 16, 8, GameComponent.SCALE);
    spriteGiant = ImageLoader.loadScaledSprite("/hero/heroL.png", 32, 12, GameComponent.SCALE);
    sprite = spriteSmall;

    //rectangle = new Rectangle(GameComponent.SCALE * 1, GameComponent.SCALE * 20, GameComponent.SCALE * 31, GameComponent.SCALE * 12);
    rectangleS = new Rectangle(0, 0, GameComponent.SCALE * 8, GameComponent.SCALE * 5);
    rectangleM = new Rectangle(0, 0, GameComponent.SCALE * 16, GameComponent.SCALE * 8);
    rectangleL = new Rectangle(0, GameComponent.SCALE, GameComponent.SCALE * 28, GameComponent.SCALE * 11);
    rectangle = rectangleS;
  }

  @Override
  public void renderMe(GameScreen screen, Level level, Hero hero) {

    if (show) {
      blitToCurrentPosition(screen, level);
    } else {
      //DO NOT SHOW
    }
    if (GameOptions.SHOW_BOUNDING) {
      BitmapLine.drawLineDDA(screen, HERO_PIXEL_X_LEFT, 0, HERO_PIXEL_X_LEFT, GameComponent.GAME_SCALED_HEIGHT - 1, CheapColor.RED);
      BitmapLine.drawLineDDA(screen, 0, heroPixelYTop, GameComponent.GAME_SCALED_WIDTH - 1, heroPixelYTop, CheapColor.RED);
    }
    drawBoundarys(screen, blitToX, blitToY, rectangle);
  }

  private void blitToCurrentPosition(GameScreen screen, Level level) {
    int currentPositionX = getPositionX();
    int currentPositionY = getPositionY();

    if ((currentPositionX <= HERO_PIXEL_X_LEFT)) {
      blitToX = currentPositionX;
    } else if (currentPositionX >= (level.totalWidthInPx - heroPixelXRight)) {
      blitToX = HERO_PIXEL_X_LEFT + (currentPositionX - (level.totalWidthInPx - heroPixelXRight));
    } else {
      blitToX = HERO_PIXEL_X_LEFT;
    }

    if (currentPositionY <= heroPixelYTop) {
      blitToY = currentPositionY;
    } else if (currentPositionY >= (level.totalHeightInPx - heroPixelYBottom)) {
      blitToY = heroPixelYTop + (currentPositionY - (level.totalHeightInPx - heroPixelYBottom));
    } else {
      blitToY = heroPixelYTop;
    }
    screen.blit(getCurrentBitmapFrame(), blitToX, blitToY);
  }

  int column      = 0;
  int line        = 0;

  int spriteCount = 1;

  private Bitmap getCurrentBitmapFrame() {
    if (velocityX < 0) {
      sprite.moveLeft();
    } else if (velocityX > 0) {
      sprite.moveRight();
    } else if (velocityY != 0) {
      sprite.spriteCount++;
    }
    sprite.checkSpriteCount();
    return sprite.currentMovingSprite();
  }

  private int     tickCountNoShown = 0;

  private boolean showHit          = false;

  private boolean show             = true;

  public void tick(Level level) {

    if (showHit) {
      tickCountNoShown++;
      if (tickCountNoShown % 12 == 0) {
        show = true;
      }
      if (tickCountNoShown % 24 == 0) {
        show = false;
      }
      if (tickCountNoShown % 109 == 0) {
        showHit = false;
        show = true;
        tickCountNoShown = 0;
      }

    } else {
      processKeys();
    }

    float nx = positionX + velocityX;
    float ny = positionY + velocityY;

    if (level.isEntityMovableTo(nx, ny, rectangle)) {
      positionX += velocityX;
      positionY += velocityY;
    }
  }

  private void processKeys() {
    if (InputKey.RIGHT.isDown) {
      velocityX += ACCELERATION_PER_TICK;
      if (velocityX >= heroTier.maxVelocity) {
        velocityX = heroTier.maxVelocity;
      }
    } else if (InputKey.LEFT.isDown) {
      velocityX -= ACCELERATION_PER_TICK;
      if (velocityX <= heroTier.minVelocity) {
        velocityX = heroTier.minVelocity;
      }
    } else {
      if (velocityX > 0.4) {
        velocityX -= 0.2f;
      }
      if (velocityX < -0.4f) {
        velocityX += 0.2f;
      } else {
        velocityX = 0.0f;
      }
    }

    if (InputKey.DOWN.isDown) {
      velocityY += ACCELERATION_PER_TICK;
      if (velocityY >= heroTier.maxVelocity) {
        velocityY = heroTier.maxVelocity;
      }
    } else if (InputKey.UP.isDown) {
      velocityY -= ACCELERATION_PER_TICK;
      if (velocityY <= heroTier.minVelocity) {
        velocityY = heroTier.minVelocity;
      }
    } else {
      if (velocityY > 0.4) {
        velocityY -= 0.2f;
      }
      if (velocityY < -0.4f) {
        velocityY += 0.2f;
      } else {
        velocityY = 0.0f;
      }
    }
  }

  public int getPositionX() {
    return (int) positionX;
  }

  public int getPositionY() {
    return (int) positionY;
  }

  public int getWidth() {
    return 32 * GameComponent.SCALE;
  }

  public int getHeight() {
    return 32 * GameComponent.SCALE;
  }

  public boolean intersectsWithEnemy(Enemy enemy) {
    Rectangle positionRecangle = new Rectangle(getPositionX() + rectangle.x, getPositionY() + rectangle.y, rectangle.width, rectangle.height);
    return positionRecangle.intersects(enemy.getPositionRectangle());
  }

  public boolean canEat(Enemy enemy) {

    if (enemy.eatableBy == null) {
      return false;
    }

    if (HeroTier.GIANT_RAT.equals(heroTier)) {
      return true;
    }

    if (HeroTier.SMALL_RAT.equals(enemy.eatableBy)) {
      return true;
    }
    if (HeroTier.MEDIUM_RAT.equals(heroTier) && HeroTier.MEDIUM_RAT.equals(enemy.eatableBy)) {
      return true;
    }
    return false;
  }

  public void eat(Enemy enemy) {
    if (!enemy.toRemove) {
      enemy.toRemove = true;
      enemiesByHungerEaten += enemy.worthHunger;
      SoundEngine.getInstance().nomSound.play();
    }
    if (health < heroTier.maxHealth) {
      health++;
    }

    if (enemiesByHungerEaten >= heroTier.maxEatToGrow) {
      velocityX = 0;
      velocityY = 0;
      SoundEngine.getInstance().upSound.play();
      if (HeroTier.SMALL_RAT.equals(heroTier)) {
        heroTier = HeroTier.MEDIUM_RAT;
        rectangle = rectangleM;
        sprite = spriteMedium;
      } else if (HeroTier.MEDIUM_RAT.equals(heroTier)) {
        heroTier = HeroTier.GIANT_RAT;
        rectangle = rectangleL;
        sprite = spriteGiant;
      } else {
        if (GameOptions.CAN_WIN) {
          GameOptions.showWin = true;
        }
      }
    }
  }

  public void hitBy(Enemy enemy) {
    if (!showHit) {
      health--;
      SoundEngine.getInstance().hitSound.play();
      showHit = true;
      tickCountNoShown = 0;
    }
    float directionX = positionX - enemy.positionX;
    float directionY = positionY - enemy.positionY;

    if (Math.abs(directionX) < 10) {
      directionX = 0;
    } else if (Math.abs(directionY) < 10) {
      directionY = 0;
    }

    Vector2D vector = new Vector2D(directionX, directionY);
    vector.normalize();

    velocityX = 0.4f * vector.x;
    velocityY = 0.4f * vector.y;
  }
}
