
package de.csw.ludum.dare.ld23.graphic.objects.enemies;

import java.awt.Rectangle;

import de.csw.ludum.dare.ld23.GameComponent;
import de.csw.ludum.dare.ld23.GameScreen;
import de.csw.ludum.dare.ld23.graphic.Bitmap;
import de.csw.ludum.dare.ld23.graphic.ImageLoader;
import de.csw.ludum.dare.ld23.graphic.Sprite;
import de.csw.ludum.dare.ld23.graphic.objects.Hero;
import de.csw.ludum.dare.ld23.graphic.objects.Hero.HeroTier;
import de.csw.ludum.dare.ld23.level.Level;
import de.csw.ludum.dare.ld23.sound.SoundEngine;

public class Bullet1 extends Enemy {

  private static final Sprite staticSprite;

  private final Sprite        sprite;

  private final Rectangle     rectangle;

  static {
    staticSprite = ImageLoader.loadScaledSprite("/enemies/bullet_1.png", 3, 3, GameComponent.SCALE, 15);
  }

  public Bullet1(int positionX, int positionY, float velocityX, float velocityY, HeroTier eatableBy) {
    this(positionX, positionY, eatableBy);
    this.velocityX = velocityX;
    this.velocityY = velocityY;
    SoundEngine.getInstance().bulletSound.play();
  }

  private Bullet1(int positionX, int positionY, HeroTier eatableBy) {
    super(positionX, positionY, null);
    sprite = Sprite.copyOf(staticSprite);
    rectangle = new Rectangle(GameComponent.SCALE * 0, GameComponent.SCALE * 0, GameComponent.SCALE * 3, GameComponent.SCALE * 3);
  }

  @Override
  public void renderMe(GameScreen screen, Level level, Hero hero) {
    int currentPositionX = getPositionX();
    int currentPositionY = getPositionY();

    int blitToX = currentPositionX + level.viewX;
    int blitToY = currentPositionY + level.viewY;

    screen.blit(getCurrentBitmapFrame(), blitToX, blitToY);
  }

  private Bitmap getCurrentBitmapFrame() {
    sprite.spriteCount++;
    sprite.checkSpriteCount();
    return sprite.currentMovingSprite();
  }

  @Override
  public void tick(Level level, Hero hero) {
    //    positionX += velocityX;
    //    positionY += velocityY;

    boolean hasMoved = moveEnemyTo(level);
    if (!hasMoved) {
      toRemove = true;
      SoundEngine.getInstance().hitSound.play();
    }
  }

  @Override
  public Rectangle getRectangle() {
    return rectangle;
  }

}
