
package de.csw.ludum.dare.ld23.graphic.objects.enemies;

import java.awt.Rectangle;

import de.csw.ludum.dare.ld23.GameComponent;
import de.csw.ludum.dare.ld23.GameScreen;
import de.csw.ludum.dare.ld23.graphic.Bitmap;
import de.csw.ludum.dare.ld23.graphic.ImageLoader;
import de.csw.ludum.dare.ld23.graphic.Sprite;
import de.csw.ludum.dare.ld23.graphic.objects.Hero;
import de.csw.ludum.dare.ld23.level.Level;
import de.csw.ludum.dare.ld23.util.GlobalRandom;
import de.csw.ludum.dare.ld23.util.Vector2D;

public class City extends Enemy {

  private static final int   FIRE_SPEED_IN_TICKS = 120;

  private static final float BULLET_SPEED        = 2.2f;

  private static final int   VISION_RANGE        = (int) Math.pow(480, 2);

  public static final int    WIDTH               = GameComponent.SCALE * 8;

  public static final int    HEIGHT              = GameComponent.SCALE * 8;

  private final Sprite       sprite;

  private final int          randomEntityInt     = GlobalRandom.nextInt(31);

  protected final Rectangle  rectangle;

  public City(int positionX, int positionY) {
    super(positionX, positionY, Hero.HeroTier.SMALL_RAT);
    sprite = ImageLoader.loadScaledSprite("/enemies/city.png", 22, 22, GameComponent.SCALE);
    rectangle = new Rectangle(GameComponent.SCALE * 0, GameComponent.SCALE * 0, GameComponent.SCALE * 22, GameComponent.SCALE * 22);
  }

  @Override
  public void renderMe(GameScreen screen, Level level, Hero hero) {
    int currentPositionX = getPositionX();
    int currentPositionY = getPositionY();

    int blitToX = currentPositionX + level.viewX;
    int blitToY = currentPositionY + level.viewY;

    screen.blit(getCurrentBitmapFrame(), blitToX, blitToY);

    drawBoundarys(screen, blitToX, blitToY, rectangle);
  }

  private Bitmap getCurrentBitmapFrame() {
    return sprite.getBitmapDirect(0, 0);
  }

  @Override
  public void tick(Level level, Hero hero) {
    if (getSquareDistanceToHero(hero) < VISION_RANGE) {
      // I'M SEEING HEROS!
      if (GameComponent.tickCount % (FIRE_SPEED_IN_TICKS + randomEntityInt) == 0) {
        fireAtHero(level, hero, 36);
      }
      if (GameComponent.tickCount % (FIRE_SPEED_IN_TICKS + randomEntityInt + 10) == 0) {
        fireAtHero(level, hero, 72);
      }
    }
  }

  private void fireAtHero(Level level, Hero hero, int offSet) {

    float posX = positionX + offSet;
    float posY = positionY + offSet;

    float directionX = hero.positionX - posX;
    float directionY = hero.positionY - posY;

    Vector2D vector = new Vector2D(directionX, directionY);
    vector.normalize();

    Bullet1 bullet = new Bullet1((int) posX, (int) posY, BULLET_SPEED * vector.x, BULLET_SPEED * vector.y, null);

    level.newEnemies.add(bullet);

  }

  @Override
  public Rectangle getRectangle() {
    return rectangle;
  }
}
