
package de.csw.ludum.dare.ld23.graphic.objects.enemies;

import java.awt.Rectangle;

import de.csw.ludum.dare.ld23.GameComponent;
import de.csw.ludum.dare.ld23.graphic.Renderable;
import de.csw.ludum.dare.ld23.graphic.objects.Hero;
import de.csw.ludum.dare.ld23.level.Level;
import de.csw.ludum.dare.ld23.util.GlobalRandom;

public abstract class Enemy extends Renderable {

  public float               positionX;

  public float               positionY;

  public float               destinationX    = 0;

  public float               destinationY    = 0;

  public long                tickCount       = 0;

  public float               velocityX       = 0.5f;

  public float               velocityY       = 0.5f;

  public boolean             toRemove        = false;

  public final int           randomEntityInt = GlobalRandom.nextInt(31);

  public final Hero.HeroTier eatableBy;

  public final long          createAt;

  public final int           worthHunger;

  public abstract void tick(Level level, Hero hero);

  public abstract Rectangle getRectangle();

  public Enemy(int positionX, int positionY, Hero.HeroTier eatableBy) {
    this.positionX = positionX;
    this.positionY = positionY;
    this.eatableBy = eatableBy;

    if (Hero.HeroTier.MEDIUM_RAT.equals(eatableBy)) {
      worthHunger = 5;
    } else if (Hero.HeroTier.GIANT_RAT.equals(eatableBy)) {
      worthHunger = 20;
    } else {
      worthHunger = 1;
    }
    createAt = System.currentTimeMillis();
  }

  protected void doRandomMovement(Level level, int randomRange) {

    if ((destinationX == 0) && (destinationY == 0) && (GameComponent.tickCount % 200 == 0)) {
      destinationX = GlobalRandom.nextInt(randomRange + 1) - (randomRange / 2);
      destinationY = GlobalRandom.nextInt(randomRange + 1) - (randomRange / 2);
      if (destinationX > 0) {
        velocityX = 0.3f;
      } else {
        velocityX = -0.3f;
      }
      if (destinationY > 0) {
        velocityY = 0.3f;
      } else {
        velocityY = -0.3f;
      }
      // System.out.println("getNewDestination " + destinationX + " : " + destinationY);
    } else if ((destinationX == 0) && (destinationY == 0)) {
      velocityX = 0;
      velocityY = 0;
    }
    if ((destinationX != 0) || (destinationY != 0)) {
      moveToDestination(level);
    }
  }

  private void moveToDestination(Level level) {
    if (destinationX != 0) {

      if (destinationX > 0) {
        destinationX--;
      } else {
        destinationX++;
      }
    }
    if (destinationY != 0) {
      if (destinationY > 0) {
        destinationY--;
      } else {
        destinationY++;
      }
    }

    moveEnemyTo(level);
  }

  protected boolean moveEnemyTo(Level level) {
    float nx = velocityX + positionX;
    float ny = velocityY + positionY;

    if (level.isEntityMovableTo(nx, ny, getRectangle())) {
      positionX += velocityX;
      positionY += velocityY;
      return true;
    } else {
      destinationX = 0;
      destinationY = 0;
      return false;
    }
  }

  public int getPositionX() {
    return (int) positionX;
  }

  public int getPositionY() {
    return (int) positionY;
  }

  public double getSquareDistanceToHero(Hero hero) {

    double distanceX = hero.positionX - positionX;
    double distanceY = hero.positionY - positionY;

    return (distanceX * distanceX) + (distanceY * distanceY);

  }

  public Rectangle getPositionRectangle() {
    Rectangle ownRect = getRectangle();
    return new Rectangle(getPositionX() + ownRect.x, getPositionY() + ownRect.y, ownRect.width, ownRect.height);
  }
}
