
package de.csw.ludum.dare.ld23.graphic.objects.enemies;

import java.awt.Rectangle;

import de.csw.ludum.dare.ld23.GameComponent;
import de.csw.ludum.dare.ld23.GameScreen;
import de.csw.ludum.dare.ld23.graphic.Bitmap;
import de.csw.ludum.dare.ld23.graphic.ImageLoader;
import de.csw.ludum.dare.ld23.graphic.Sprite;
import de.csw.ludum.dare.ld23.graphic.objects.Hero;
import de.csw.ludum.dare.ld23.level.Level;

public class Kitten extends Enemy {

  public static final int   WIDTH  = GameComponent.SCALE * 8;

  public static final int   HEIGHT = GameComponent.SCALE * 8;

  private final Sprite      sprite;

  protected final Rectangle rectangle;

  public Kitten(int positionX, int positionY) {
    super(positionX, positionY, Hero.HeroTier.SMALL_RAT);
    sprite = ImageLoader.loadScaledSprite("/enemies/kitten2.png", 5, 6, GameComponent.SCALE);
    rectangle = new Rectangle(GameComponent.SCALE * 0, GameComponent.SCALE * 0, GameComponent.SCALE * 5, GameComponent.SCALE * 6);
  }

  @Override
  public void renderMe(GameScreen screen, Level level, Hero hero) {
    int currentPositionX = getPositionX();
    int currentPositionY = getPositionY();

    int blitToX = currentPositionX + level.viewX;
    int blitToY = currentPositionY + level.viewY;

    screen.blit(getCurrentBitmapFrame(), blitToX, blitToY);

    drawBoundarys(screen, blitToX, blitToY, rectangle);
  }

  private Bitmap getCurrentBitmapFrame() {
    if (velocityX < 0) {
      sprite.moveLeft();
    } else if (velocityX > 0) {
      sprite.moveRight();
    } else if (velocityY != 0) {
      sprite.spriteCount++;
    }
    sprite.checkSpriteCount();
    return sprite.currentMovingSprite();
  }

  @Override
  public void tick(Level level, Hero hero) {

    if (getSquareDistanceToHero(hero) < 22500) {
      // I'M SEEING HEROS!

      if (hero.positionX - positionX > 10) {
        velocityX = -0.3f;
      } else if (hero.positionX - positionX < -10) {
        velocityX = 0.3f;
      } else {
        velocityX = 0.0f;
      }
      if (hero.positionY - positionY > 10) {
        velocityY = -0.3f;
      } else if (hero.positionY - positionY < -10) {
        velocityY = 0.3f;
      } else {
        velocityY = 0.0f;
      }

      moveEnemyTo(level);
    } else {
      doRandomMovement(level, 140);
    }
  }

  @Override
  public Rectangle getRectangle() {
    return rectangle;
  }
}
