
package de.csw.ludum.dare.ld23.graphic.objects.enemies;

import java.awt.Rectangle;

import de.csw.ludum.dare.ld23.GameComponent;
import de.csw.ludum.dare.ld23.GameScreen;
import de.csw.ludum.dare.ld23.graphic.Bitmap;
import de.csw.ludum.dare.ld23.graphic.ImageLoader;
import de.csw.ludum.dare.ld23.graphic.Sprite;
import de.csw.ludum.dare.ld23.graphic.objects.Hero;
import de.csw.ludum.dare.ld23.level.Level;
import de.csw.ludum.dare.ld23.util.Vector2D;

public class Soldier1 extends Enemy {

  private static final int   FIRE_SPEED_IN_TICKS = 200;

  private static final float BULLET_SPEED        = 1.1f;

  private static final int   VISION_RANGE        = (int) Math.pow(330, 2);

  public static final int    WIDTH               = GameComponent.SCALE * 8;

  public static final int    HEIGHT              = GameComponent.SCALE * 8;

  private final Sprite       sprite;

  private Rectangle          rectangle;

  public Soldier1(int positionX, int positionY) {
    super(positionX, positionY, Hero.HeroTier.MEDIUM_RAT);
    sprite = ImageLoader.loadScaledSprite("/enemies/soldier_1.png", 8, 8, GameComponent.SCALE, 5);
    rectangle = new Rectangle(GameComponent.SCALE * 0, GameComponent.SCALE * 0, GameComponent.SCALE * 8, GameComponent.SCALE * 8);
  }

  @Override
  public void renderMe(GameScreen screen, Level level, Hero hero) {

    int blitToX = getPositionX() + level.viewX;
    int blitToY = getPositionY() + level.viewY;

    screen.blit(getCurrentBitmapFrame(), blitToX, blitToY);

    drawBoundarys(screen, blitToX, blitToY, rectangle);
  }

  private Bitmap getCurrentBitmapFrame() {
    if (velocityX < 0) {
      sprite.moveLeft();
    } else if (velocityX > 0) {
      sprite.moveRight();
    } else if (velocityY != 0) {
      sprite.spriteCount++;
    }
    sprite.checkSpriteCount();
    return sprite.currentMovingSprite();
  }

  @Override
  public void tick(Level level, Hero hero) {
    if (getSquareDistanceToHero(hero) < VISION_RANGE) {
      // I'M SEEING HEROS!
      if (GameComponent.tickCount % (FIRE_SPEED_IN_TICKS + randomEntityInt) == 0) {
        fireAtHero(level, hero);
      }
    } else {
      doRandomMovement(level, 100);
    }
  }

  private void fireAtHero(Level level, Hero hero) {

    float posX = positionX + 4;
    float posY = positionY + 4;

    float directionX = hero.positionX - posX;
    float directionY = hero.positionY - posY;

    Vector2D vector = new Vector2D(directionX, directionY);
    vector.normalize();

    Bullet1 bullet = new Bullet1((int) posX, (int) posY, BULLET_SPEED * vector.x, BULLET_SPEED * vector.y, null);

    level.newEnemies.add(bullet);

  }

  @Override
  public Rectangle getRectangle() {
    return rectangle;
  }
}
