
package de.csw.ludum.dare.ld23.graphic.objects.enemies;

import java.awt.Rectangle;

import de.csw.ludum.dare.ld23.GameComponent;
import de.csw.ludum.dare.ld23.GameScreen;
import de.csw.ludum.dare.ld23.graphic.Bitmap;
import de.csw.ludum.dare.ld23.graphic.ImageLoader;
import de.csw.ludum.dare.ld23.graphic.Sprite;
import de.csw.ludum.dare.ld23.graphic.objects.Hero;
import de.csw.ludum.dare.ld23.level.Level;
import de.csw.ludum.dare.ld23.util.Vector2D;

public class Tank1 extends Enemy {

  private static final int   FIRE_SPEED_IN_TICKS = 250;

  private static final float BULLET_SPEED        = 1.6f;

  private static final int   VISION_RANGE        = (int) Math.pow(400, 2);

  public static final int    WIDTH               = GameComponent.SCALE * 20;

  public static final int    HEIGHT              = GameComponent.SCALE * 20;

  private final Sprite       sprite;

  private Rectangle          rectangle;

  public Tank1(int positionX, int positionY) {
    super(positionX, positionY, Hero.HeroTier.GIANT_RAT);
    sprite = ImageLoader.loadScaledSprite("/enemies/tank_1.png", 20, 20, GameComponent.SCALE);
    rectangle = new Rectangle(GameComponent.SCALE * 0, GameComponent.SCALE * 0, GameComponent.SCALE * 20, GameComponent.SCALE * 20);
  }

  @Override
  public void renderMe(GameScreen screen, Level level, Hero hero) {
    int blitToX = getPositionX() + level.viewX;
    int blitToY = getPositionY() + level.viewY;

    screen.blit(getCurrentBitmapFrame(), blitToX, blitToY);

    drawBoundarys(screen, blitToX, blitToY, rectangle);
  }

  private Bitmap getCurrentBitmapFrame() {
    return sprite.getBitmapDirect(0, 0);
  }

  @Override
  public void tick(Level level, Hero hero) {
    if (getSquareDistanceToHero(hero) < VISION_RANGE) {
      // I'M SEEING HEROS!
      if (GameComponent.tickCount % (FIRE_SPEED_IN_TICKS + randomEntityInt) == 0) {
        fireAtHero(level, hero, 4);
      }
      if (GameComponent.tickCount % (FIRE_SPEED_IN_TICKS + randomEntityInt + 40) == 0) {
        fireAtHero(level, hero, 4);
      }
    } else {
      doRandomMovement(level, 30);
    }
  }

  private void fireAtHero(Level level, Hero hero, int offSet) {

    float posX = positionX + offSet;
    float posY = positionY + offSet;

    float directionX = hero.positionX - posX;
    float directionY = hero.positionY - posY;

    Vector2D vector = new Vector2D(directionX, directionY);
    vector.normalize();

    Bullet1 bullet = new Bullet1((int) posX, (int) posY, BULLET_SPEED * vector.x, BULLET_SPEED * vector.y, null);

    level.newEnemies.add(bullet);

  }

  @Override
  public Rectangle getRectangle() {
    return rectangle;
  }
}
