
package de.csw.ludum.dare.ld23.level;

import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.csw.ludum.dare.ld23.GameComponent;
import de.csw.ludum.dare.ld23.GameOptions;
import de.csw.ludum.dare.ld23.GameScreen;
import de.csw.ludum.dare.ld23.graphic.ImageLoader;
import de.csw.ludum.dare.ld23.graphic.Sprite;
import de.csw.ludum.dare.ld23.graphic.Tile;
import de.csw.ludum.dare.ld23.graphic.objects.Hero;
import de.csw.ludum.dare.ld23.graphic.objects.enemies.City;
import de.csw.ludum.dare.ld23.graphic.objects.enemies.Enemy;
import de.csw.ludum.dare.ld23.graphic.objects.enemies.House;
import de.csw.ludum.dare.ld23.graphic.objects.enemies.Human;
import de.csw.ludum.dare.ld23.graphic.objects.enemies.Kitten;
import de.csw.ludum.dare.ld23.graphic.objects.enemies.Soldier1;
import de.csw.ludum.dare.ld23.graphic.objects.enemies.Tank1;
import de.csw.ludum.dare.ld23.util.BitmapLine;
import de.csw.ludum.dare.ld23.util.CheapColor;

public class Level {

  private static final int TILE_SIZE_IN_PX        = 16;

  private static final int SCALED_TILE_SIZE_IN_PX = GameComponent.SCALE * TILE_SIZE_IN_PX;

  char[][]                 levelDesc;

  Tile[][]                 levelDescTilesPerLineToColumn;

  // 20x12

  public int               viewX                  = 0;

  public int               viewY                  = 0;

  private final Sprite     sprite;

  // private final Sprite levelFormat;

  //private Tile[]       tiles;

  public final int         countTilesX;
  public final int         countTilesY;

  public final int         totalWidthInPx;
  public final int         totalHeightInPx;

  public final int         NEGATIVE_MAX_VIEW_X_WIDTH;
  public final int         NEGATIVE_MAX_VIEW_Y;

  public List<Enemy>       enemies                = Collections.synchronizedList(new ArrayList<Enemy>());

  public List<Enemy>       newEnemies             = new ArrayList<Enemy>();

  public Level() {
    sprite = ImageLoader.loadSprite("/floor_tiles.png", TILE_SIZE_IN_PX, TILE_SIZE_IN_PX, 0, 0);

    //levelFormat = ImageLoader.loadSprite("/level1.png", 100, 40, 0, 0);

    //    sprite.getBitmapDirect(nr, 0);
    //tiles = new Tile[4];

    //    tiles[0] = new Tile(sprite.getBitmapDirect(0, 0), false, '#');
    //    tiles[1] = new Tile(sprite.getBitmapDirect(1, 0), true, ' ');

    // K = Kitten
    // T = Tank
    // S = Soldier
    // C = City
    // H = House
    // P = Human

    List<String> levelInAscii = new ArrayList<String>();
    levelInAscii.add("                                                                  ");
    levelInAscii.add("                                                                  ");
    levelInAscii.add("                           0 1 1 1 1 1 2                          ");
    levelInAscii.add("                         0 5 5 5 5 5S5 5 2                        ");
    levelInAscii.add("                     0 1 5 5 5K5 5 5 5S5 5 1 2                    ");
    levelInAscii.add("                   0 5 5 5KB 5 5 B B D D D 5 5 2                  ");
    levelInAscii.add("                 0 5 5 5 5 5 B 5 5SD D C D 5S5 5 2                ");
    levelInAscii.add("               0 5 5 5 5K5P5 5 5 5 5 D DSD 5 5 B 5 2              ");
    levelInAscii.add("               4 5 5 5P5H5 B 5 5 5 5 B 5 D B 5 5 B 6              ");
    levelInAscii.add("             0 B 5 5K5 B 5 5 5 5 B 5 5S5 5H5 5S5 5 5 2            ");
    levelInAscii.add("           0 B 5 5 5 B 5 5K5 5 5 B 5S5 5T5 B 5 5T5 B 5 2          ");
    levelInAscii.add("         0 5 5 5 5 B 5 5 5 5 5 # B 5 B 5 B 5 5 5 5 5 5T5 2        ");
    levelInAscii.add("         4 3 5 5K5 5 5 5 B 5 5 # 5 5 5 5 5P5 5 5 5 5 5 5 6        ");
    levelInAscii.add("       0 5 5 5 B 5 5K5 B 5 5 5 # B B 5 5T5HB B B B B B 5 5 2      ");
    levelInAscii.add("     0 5 5 5 5 5S5 5 5 5 5K5 #S5 5 B 5 5H5H5 5 5 5 5 5P5 5S5 2    ");
    levelInAscii.add("     4 5P5 5 5 B 5 5K5 B 5 5 # 5 5S5 5 5P5P5P5 5 5 5P5C5P5S5 6    ");
    levelInAscii.add("     4 5P5P5K5 5 5 5 5 5 5 5 # 5 5 5 5 5P5P5 5 5 3 5P5P5 5 5 6    ");
    levelInAscii.add("     4 5H5P5 5 5 B 5 B 5P5 5 # 5 5 B 5S5 5 5K5 5 5 5 5 5 5 5 6    ");
    levelInAscii.add("     8 5 5S5K5 5 5K5 5 5H5 5 # 5 B 5 5 5 5 5 5 B 5 5 5 5 5 5 A    ");
    levelInAscii.add("       8 5P5 5 B 5 5 5 5 5 5 5 # 5 5 5K5 3 5 5 5 5 5 5 5S5 A      ");
    levelInAscii.add("         4 5 5S5 5 5 5 5K5 5 B # 5 B 5S5 5 5 5P5 5 5 5 5 6        ");
    levelInAscii.add("         8 5 5 B 5 5 5 5 5 5 5 # 5 5 5 5 5S5 5H5 5 5 5 5 A        ");
    levelInAscii.add("           8 5 5K5 5K5 5 5K5 5 B 5 5 5K5 B 5 5P5 5 5 5 A          ");
    levelInAscii.add("             8 5 B 5 5 5K5 5 5 5 5 5S5 5 5 5 5S5 5 5 A            ");
    levelInAscii.add("               4 5 5K5K5P5 5P5 5 5 5 5 5 5 5 5 5 5 6              ");
    levelInAscii.add("               8 B 5S5P5H5H5 B 5 5P5H5 5 5S5 B 5 5 A              ");
    levelInAscii.add("                 8 5 5HBP5H5 5 5HBK5 5 5 5 B 5 5 A                ");
    levelInAscii.add("                   8 5 5P5 B 5 5P5 B B 5 5 5 5 A                  ");
    levelInAscii.add("                     8 9 5 5 3 5 5S5 5 5T5 9 A                    ");
    levelInAscii.add("                         8 5S5 5 5 5 5 5 A                        ");
    levelInAscii.add("                           8 9 9 9 9 9 A                          ");
    levelInAscii.add("                                                                  ");
    levelInAscii.add("                                                                  ");

    countTilesX = levelInAscii.get(0).length() / 2;
    countTilesY = levelInAscii.size();

    createEnemies(levelInAscii);

    totalWidthInPx = countTilesX * SCALED_TILE_SIZE_IN_PX;
    totalHeightInPx = countTilesY * SCALED_TILE_SIZE_IN_PX;

    NEGATIVE_MAX_VIEW_X_WIDTH = -1 * (totalWidthInPx - Hero.heroPixelXRight - Hero.HERO_PIXEL_X_LEFT);
    NEGATIVE_MAX_VIEW_Y = -1 * (totalHeightInPx - Hero.heroPixelYBottom - Hero.heroPixelYTop);

    levelDesc = new char[countTilesY][countTilesX];// { {  } };
    levelDescTilesPerLineToColumn = new Tile[countTilesY][countTilesX];;

    for (int line = 0; line < countTilesY; line++) {
      String asciiLine = levelInAscii.get(line);
      for (int charOfLine = 0; charOfLine < countTilesX; charOfLine++) {
        levelDesc[line][charOfLine] = asciiLine.charAt(charOfLine * 2 + 1);
        levelDescTilesPerLineToColumn[line][charOfLine] = createTile(asciiLine.charAt(charOfLine * 2 + 1), line, charOfLine, sprite);
      }
    }
  }

  private void createEnemies(List<String> levelInAscii) {
    // K = Kitten
    // T = Tank
    // S = Soldier
    // C = City
    // H = House
    // P = Human
    for (int line = 0; line < countTilesY; line++) {
      String asciiLine = levelInAscii.get(line);
      for (int charOfLine = 0; charOfLine < countTilesX; charOfLine++) {
        char c = asciiLine.charAt(charOfLine * 2);
        if (c != ' ') {
          int positionX = charOfLine * TILE_SIZE_IN_PX * GameComponent.SCALE;
          int positionY = line * TILE_SIZE_IN_PX * GameComponent.SCALE;

          //          System.out.println("Create Enemy to " + positionX + " : " + positionY);

          if (c == 'K') {
            enemies.add(new Kitten(positionX, positionY));
          }
          if (c == 'T') {
            enemies.add(new Tank1(positionX, positionY));
          }
          if (c == 'S') {
            enemies.add(new Soldier1(positionX, positionY));
          }
          if (c == 'C') {
            enemies.add(new City(positionX, positionY));
          }
          if (c == 'H') {
            enemies.add(new House(positionX, positionY));
          }
          if (c == 'P') {
            enemies.add(new Human(positionX, positionY));
          }
        }
      }
    }
  }

  private Tile createTile(char charAt, int line, int charOfLine, Sprite sprite2) {
    int scale = GameComponent.SCALE;

    //    System.out.println(rect);
    int rectX = charOfLine * SCALED_TILE_SIZE_IN_PX;
    int rectY = line * SCALED_TILE_SIZE_IN_PX;
    int tileSize = SCALED_TILE_SIZE_IN_PX;
    int halfTileSize = tileSize / 2;

    if (charAt == '#') {
      Rectangle rect = new Rectangle(rectX, rectY, tileSize, tileSize);
      return new Tile(sprite.getBitmapDirect(7), false, charAt, rect, scale);
    } else if (charAt == '/') {
      Rectangle rect = new Rectangle(rectX, rectY, tileSize, tileSize);
      return new Tile(sprite.getBitmapDirect(13), true, charAt, rect, scale);
    } else if (charAt == 'G') {
      Rectangle rect = new Rectangle(rectX, rectY, tileSize, tileSize);
      return new Tile(sprite.getBitmapDirect(14), true, charAt, rect, scale);
    } else if (charAt == '0') {
      Rectangle rect = new Rectangle(rectX, rectY, halfTileSize, halfTileSize);
      return new Tile(sprite.getBitmapDirect(0), false, charAt, rect, scale);
    } else if (charAt == '1') {
      Rectangle rect = new Rectangle(rectX, rectY, tileSize, tileSize);
      return new Tile(sprite.getBitmapDirect(1), true, charAt, rect, scale);
    } else if (charAt == '2') {
      Rectangle rect = new Rectangle(rectX + halfTileSize, rectY, halfTileSize, halfTileSize);
      return new Tile(sprite.getBitmapDirect(2), false, charAt, rect, scale);
    } else if (charAt == '3') {
      Rectangle rect = new Rectangle(rectX, rectY, tileSize, tileSize);
      return new Tile(sprite.getBitmapDirect(3), true, charAt, rect, scale);
    } else if (charAt == '4') {
      Rectangle rect = new Rectangle(rectX, rectY, tileSize, tileSize);
      return new Tile(sprite.getBitmapDirect(4), true, charAt, rect, scale);
    } else if (charAt == '5') {
      Rectangle rect = new Rectangle(rectX, rectY, tileSize, tileSize);
      return new Tile(sprite.getBitmapDirect(5), true, charAt, rect, scale);
    } else if (charAt == '6') {
      Rectangle rect = new Rectangle(rectX, rectY, tileSize, tileSize);
      return new Tile(sprite.getBitmapDirect(6), true, charAt, rect, scale);
    } else if (charAt == '7') {
      Rectangle rect = new Rectangle(rectX, rectY, tileSize, tileSize);
      return new Tile(sprite.getBitmapDirect(7), true, charAt, rect, scale);
    } else if (charAt == '8') {
      Rectangle rect = new Rectangle(rectX, rectY + halfTileSize, halfTileSize, halfTileSize);
      return new Tile(sprite.getBitmapDirect(8), false, charAt, rect, scale);
    } else if (charAt == '9') {
      Rectangle rect = new Rectangle(rectX, rectY, tileSize, tileSize);
      return new Tile(sprite.getBitmapDirect(9), true, charAt, rect, scale);
    } else if (charAt == 'A') {
      Rectangle rect = new Rectangle(rectX + halfTileSize, rectY + halfTileSize, halfTileSize, halfTileSize);
      return new Tile(sprite.getBitmapDirect(10), false, charAt, rect, scale);
    } else if (charAt == 'B') {
      Rectangle rect = new Rectangle(rectX, rectY, tileSize, tileSize);
      return new Tile(sprite.getBitmapDirect(11), true, charAt, rect, scale);
    } else if (charAt == 'C') {
      Rectangle rect = new Rectangle(rectX, rectY, tileSize, tileSize);
      return new Tile(sprite.getBitmapDirect(12), true, charAt, rect, scale);
    } else if (charAt == 'D') {
      Rectangle rect = new Rectangle(rectX, rectY, tileSize, tileSize);
      return new Tile(sprite.getBitmapDirect(13), true, charAt, rect, scale);
    } else if (charAt == 'E') {
      Rectangle rect = new Rectangle(rectX, rectY, tileSize, tileSize);
      return new Tile(sprite.getBitmapDirect(14), true, charAt, rect, scale);
    } else if (charAt == 'F') {
      Rectangle rect = new Rectangle(rectX, rectY, tileSize, tileSize);
      return new Tile(sprite.getBitmapDirect(15), true, charAt, rect, scale);
    }

    return null;
  }

  public void renderMe(GameScreen screen, Hero hero) {

    if (hero.getPositionX() >= Hero.HERO_PIXEL_X_LEFT) {
      viewX = -1 * (hero.getPositionX() - Hero.HERO_PIXEL_X_LEFT);
      if (viewX < NEGATIVE_MAX_VIEW_X_WIDTH) {
        viewX = NEGATIVE_MAX_VIEW_X_WIDTH;
      }
    } else {
      viewX = 0;
    }

    if (hero.getPositionY() >= hero.heroPixelYTop) {
      viewY = -1 * (hero.getPositionY() - hero.heroPixelYTop);
      if (viewY < NEGATIVE_MAX_VIEW_Y) {
        viewY = NEGATIVE_MAX_VIEW_Y;
      }
    } else {
      viewY = 0;
    }

    for (int line = 0; line < countTilesY; line++) {
      for (int charOfLine = 0; charOfLine < countTilesX; charOfLine++) {
        Tile t = levelDescTilesPerLineToColumn[line][charOfLine];
        drawTileTo(screen, t, charOfLine, line);
      }
    }

  }

  private void drawTileTo(GameScreen screen, Tile tile1, int tilePosInTilesX, int tilePosInTilesY) {
    if (tile1 != null) {
      int tilePosX = tilePosInTilesX * SCALED_TILE_SIZE_IN_PX;
      int tilePosY = tilePosInTilesY * SCALED_TILE_SIZE_IN_PX;

      screen.blit(tile1.bitmap, tilePosX + viewX, tilePosY + viewY);

      if (GameOptions.SHOW_BOUNDING) {
        BitmapLine.drawRectangle(screen, tile1.rectangle, CheapColor.RED);
      }
    }
  }

  public boolean isEntityMovableTo(float nx, float ny, Rectangle entityRectangle) {

    Rectangle newBoundaryRectangle = new Rectangle(entityRectangle.x + (int) nx, entityRectangle.y + (int) ny, entityRectangle.width,
        entityRectangle.height);

    Tile[] maybeIntersetingTiles = getMaybeIntersetingTiles(newBoundaryRectangle);

    if (maybeIntersetingTiles[0] == null) {
      return false;
    }
    for (Tile tile : maybeIntersetingTiles) {
      if ((tile != null) && (!tile.isWalkable())) {
        boolean isIntersecting = tile.isIntersecting(newBoundaryRectangle);
        if (isIntersecting) {
          return false;
        }
      }
    }

    return true;
  }

  private Tile[] getMaybeIntersetingTiles(Rectangle newBoundaryRectangle) {

    int tile1IndexX = newBoundaryRectangle.x / SCALED_TILE_SIZE_IN_PX;
    int tile1IndexY = newBoundaryRectangle.y / SCALED_TILE_SIZE_IN_PX;

    // Upper: Left and Right
    Tile tile1 = getTile(tile1IndexX, tile1IndexY);
    Tile tile2 = getTile(tile1IndexX + 1, tile1IndexY);

    // MIDDLE: Left and Right
    Tile tile3 = getTile(tile1IndexX, tile1IndexY + 1);
    Tile tile4 = getTile(tile1IndexX + 1, tile1IndexY + 1);

    Tile tile5 = getTile(tile1IndexX, tile1IndexY + 2);
    Tile tile6 = getTile(tile1IndexX + 1, tile1IndexY + 2);

    Tile tile7 = getTile(tile1IndexX - 1, tile1IndexY - 1);
    Tile tile8 = getTile(tile1IndexX - 1, tile1IndexY);
    Tile tile9 = getTile(tile1IndexX, tile1IndexY - 1);
    return new Tile[] { tile1, tile2, tile3, tile4, tile5, tile6, tile7, tile8, tile9 };
  }

  private Tile getTile(int tileIndexX, int tileIndexY) {
    if ((tileIndexX >= countTilesX) || (tileIndexY >= countTilesY) || (tileIndexX < 0) || (tileIndexY < 0)) {
      return null;
    }
    return levelDescTilesPerLineToColumn[tileIndexY][tileIndexX];
  }

  public void tick(Hero hero) {
    // TODO Do Anything?
    // TODO get new Koordinates
    // 960 / 2 = 480 + (16*SCALE)
    //    int heroPixelXMiddle = GameComponent.GAME_SCALED_WIDTH / 2 + (hero.getWidth() / 2);
    //
    //    if(true)

  }
}
