
package de.csw.ludum.dare.ld23.sound;

import java.io.IOException;
import java.io.InputStream;

import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.FloatControl;
import javax.sound.sampled.LineEvent;
import javax.sound.sampled.LineEvent.Type;
import javax.sound.sampled.LineListener;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;

import de.csw.ludum.dare.ld23.sound.SoundEngine.Volume;
import de.csw.ludum.dare.ld23.util.DebugLogger;

public class SoundEffect {

  private final Clip   clip;

  private final String fullPath;

  private boolean      loop;

  private boolean      playing = false;

  private Volume       volume;

  protected SoundEffect() {
    fullPath = null;
    clip = null;
  }

  public SoundEffect(String fileFullPath, Volume volume, boolean toLoop) throws LineUnavailableException, IOException, UnsupportedAudioFileException {

    this.volume = volume;
    loop = toLoop;
    fullPath = fileFullPath;

    InputStream is = SoundEffect.class.getResourceAsStream(fileFullPath);
    AudioInputStream audioIn = AudioSystem.getAudioInputStream(is);
    clip = AudioSystem.getClip();
    clip.open(audioIn);

    adujustVolumne();

    clip.addLineListener(new LineListener() {

      @Override
      public void update(LineEvent event) {
        Type type = event.getType();
        DebugLogger.logInfo(fullPath + " changed to " + type);
        if (Type.STOP.equals(type) && playing && loop) {
          play();
        }
      }
    });
  }

  private void adujustVolumne() {
    if (Volume.LOW.equals(volume)) {
      FloatControl gainControl = (FloatControl) clip.getControl(FloatControl.Type.MASTER_GAIN);
      gainControl.setValue(-8.0f);
    }
    if (Volume.HIGH.equals(volume)) {
      FloatControl gainControl = (FloatControl) clip.getControl(FloatControl.Type.MASTER_GAIN);
      gainControl.setValue(+8.0f);
    }
  }

  public void play() {
    playing = true;
    SoundEngine.getInstance().playSound(clip);
  }

  public void stop() {
    clip.stop();
    playing = false;
  }
}
