
package de.csw.ludum.dare.ld23.sound;

import java.io.IOException;

import javax.sound.sampled.Clip;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;

import de.csw.ludum.dare.ld23.GameOptions;
import de.csw.ludum.dare.ld23.util.DebugLogger;

// Singleton
public class SoundEngine {

  private static volatile SoundEngine soundEngine;

  public SoundEffect                  nomSound;

  public SoundEffect                  bulletSound;

  public SoundEffect                  enterSound;

  public SoundEffect                  hitSound;

  public SoundEffect                  upSound;

  public SoundEffect                  song1;

  public static enum Volume {
    MUTE,
    LOW,
    MEDIUM,
    HIGH
  }

  public Volume volume = Volume.LOW;

  public void init() {
    try {
      nomSound.stop();
      bulletSound.stop();
      song1.stop();
      enterSound.stop();
      hitSound.stop();
      upSound.stop();
    } catch (Exception e) {
      System.out.println("MOCKING SOUND ENGINE IN INIT!");
      soundEngine = new SoundEngineMock();
    }

    try {
      Thread.sleep(50);
    } catch (InterruptedException e) {
      e.printStackTrace();
    }
  }

  public static SoundEngine getInstance() {
    try {
      if (soundEngine == null) {
        DebugLogger.logInfoWithTimeAndThread("Creating SoundEngine...");
        soundEngine = new SoundEngine();
        DebugLogger.logInfoWithTimeAndThread("SoundEngine created.");
      }
    } catch (Throwable e) {
      e.printStackTrace();
      System.out.println("MOCKING SOUND ENGINE GET_INSTANCE!");
      soundEngine = new SoundEngineMock();
    }
    return soundEngine;
  }

  protected SoundEngine() {
    try {
      nomSound = new SoundEffect("/sound/nom.wav", Volume.LOW, false);
      bulletSound = new SoundEffect("/sound/bullet.wav", Volume.LOW, false);
      song1 = new SoundEffect("/sound/song.wav", Volume.LOW, true);
      enterSound = new SoundEffect("/sound/enter.wav", Volume.MEDIUM, false);
      hitSound = new SoundEffect("/sound/hit.wav", Volume.MEDIUM, false);
      upSound = new SoundEffect("/sound/up.wav", Volume.MEDIUM, false);
    } catch (LineUnavailableException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    } catch (UnsupportedAudioFileException e) {
      e.printStackTrace();
    }
  }

  public static void muteOrUnmute() {
    if (Volume.MUTE.equals(getInstance().volume)) {
      getInstance().volume = Volume.LOW;
      getInstance().song1.play();
    } else {
      getInstance().volume = Volume.MUTE;
      getInstance().song1.stop();
    }
  }

  public boolean isMuted() {
    return volume == Volume.MUTE;
  }

  public void playSound(Clip clip) {
    if (!isMuted() && GameOptions.playMusic) {
      if (clip.isRunning()) {
        clip.stop();
      }
      clip.setFramePosition(0);
      clip.start();
    }
  }

}
