
package de.csw.ludum.dare.ld23.util;

import de.csw.ludum.dare.ld23.graphic.Bitmap;

public class BitmapTransform {

  public static Bitmap resize(Bitmap bitmap, int scale) {
    int[] resize = BitmapTransform.resize(bitmap.pixels, bitmap.width, bitmap.height, scale * bitmap.width, scale * bitmap.height);
    return new Bitmap(scale * bitmap.width, scale * bitmap.height, resize);
  }

  public static int[] resize(int[] pixels, int widthSource, int heightSource, int width, int height) {
    int[] pd = new int[width * height];
    float xs = (float) widthSource / width;
    float ys = (float) heightSource / height;

    float sx, sy;
    int x0, y0;

    // Nearest Neighbor
    int srcIdx = 0;
    for (int y = 0; y < height; y++) {
      for (int x = 0; x < width; x++) {
        sx = x * xs;
        sy = y * ys;
        x0 = (int) sx;
        y0 = (int) sy;

        pd[srcIdx++] = pixels[y0 * widthSource + x0];
      }
    }

    return pd;
  }

  public static Bitmap editAndScaleBitmap(Bitmap bitmap, int scale) {
    Bitmap resizedBitmap = BitmapTransform.resize(bitmap, scale);

    PixelOps.darkenBottomHalfOfPixels(resizedBitmap, bitmap.width, bitmap.height, scale);
    PixelOps.darkenColumns(resizedBitmap, bitmap.width, bitmap.height, scale);
    PixelOps.randomDarkenMetaPixel(resizedBitmap, bitmap.width, bitmap.height, scale);
    PixelOps.randomDarkenPixels(resizedBitmap);
    //    PixelOps.asdfDarkenColumns();
    //    PixelOps.asdfDarkenLines();
    //    PixelOps.asdfRandomTonePixels();

    return resizedBitmap;
  }

  public static void randomPixelColors(Bitmap bitmap) {
    for (int i = 0; i < bitmap.pixels.length; i++) {
      bitmap.pixels[i] = randomColor(bitmap.pixels[i]);
    }
  }

  private static int colorRandomValue = 20;

  private static int randomColor(int pixel) {

    int a = (pixel >>> 24) & 0x000000FF;
    int r = (pixel >>> 16) & 0x000000FF;
    int g = (pixel >>> 8) & 0x000000FF;
    int b = (pixel) & 0x000000FF;

    int substractValue = GlobalRandom.nextInt(colorRandomValue) - (colorRandomValue / 2);
    r = PixelOps.darkenColorTone(r, substractValue);

    substractValue = GlobalRandom.nextInt(colorRandomValue) - (colorRandomValue / 2);
    g = PixelOps.darkenColorTone(g, substractValue);

    substractValue = GlobalRandom.nextInt(colorRandomValue) - (colorRandomValue / 2);
    b = PixelOps.darkenColorTone(b, substractValue);

    return CheapColor.colorOfARGB(a, r, g, b);
  }
}
